import { PromiseCapability } from './utils'

export class PDFFetchStream {
  fetchCapability: PromiseCapability<any>
  url: string
  isHttp: boolean
  _headers: Headers
  _withCredentials: boolean
  _abortController: AbortController

  constructor(options: {
    url: string,
    httpHeaders?: any,
    withCredentials?: boolean,
  }) {
    this.fetchCapability = new PromiseCapability()
    this.url = options.url
    this.isHttp = /^https?:/i.test(options.url)
    this._headers = this.createHeaders(options.httpHeaders)
    this._abortController = new AbortController()
    this._withCredentials = options.withCredentials || false
  }

  createHeaders(httpHeaders: Record<string, string | undefined>): Headers {
    httpHeaders = (this.isHttp && httpHeaders) || {}
    const headers = new Headers()

    for (const property in httpHeaders) {
      const value = httpHeaders[property]
      if (value === undefined) {
        continue
      }
      headers.append(property, value)
    }
    return headers
  }

  fetchPDF() {
    const url = this.url
    const fetchOptions = this.createFetchOptions()
    const fetchCapability = this.fetchCapability

    fetch(
      url,
      fetchOptions
    )
      .then(response => {
        if (!this.validateResponseStatus(response.status)) {
          fetchCapability.reject(this.createResponseStatusError(response.status, url))
        }
        const type = response.headers.get("Content-Type")
        if (type?.startsWith('application/json')) {
          const errorObj = response.json();
          fetchCapability.reject({
            message: errorObj
          })
        } else {
          fetchCapability.resolve(response.blob())
        }
      })
      .catch(() => {
        this.fetchCapability.reject({
          message: "Network or Cross Domain Access Error."
        })
      })
    
    return fetchCapability.promise
  }

  createFetchOptions(): RequestInit {
    const { _headers: headers, _withCredentials: withCredentials, _abortController: abortController } = this
    return {
      method: "GET",
      headers,
      signal: abortController.signal,
      mode: "cors",
      credentials: withCredentials ? "include" : "same-origin",
      redirect: "follow",
    }
  }

  validateResponseStatus(status: number) {
    return status === 200 || status === 206
  }

  createResponseStatusError(status: number, url: string) {
    if (status === 404 || (status === 0 && url.startsWith("file:"))) {
      return {
        name: 'Missing PDF File',
        message: 'Missing PDF "' + url + '".'
      }
    }
    return {
      status,
      message: `Unexpected server response (${status}) while retrieving PDF "${url}".`
    }
  }
}
